@@ -1,14 +1,11 @@
 from .__version__ import __version__
-from .mocks import MockTransport
+from .mocks import DeprecatedMockTransport as MockTransport
 from .models import MockResponse, Route
 from .router import Router
-from .transports import AsyncMockTransport, SyncMockTransport
 
 from .api import (  # isort:skip
     mock,
     routes,
-    aliases,
-    stats,
     calls,
     start,
     stop,
@@ -17,6 +14,7 @@ from .api import (  # isort:skip
     pop,
     route,
     add,
+    request,
     get,
     post,
     put,
@@ -29,15 +27,11 @@ from .api import (  # isort:skip
 __all__ = [
     "__version__",
     "MockTransport",
-    "AsyncMockTransport",
-    "SyncMockTransport",
+    "MockResponse",
     "Router",
     "Route",
-    "MockResponse",
     "mock",
     "routes",
-    "aliases",
-    "stats",
     "calls",
     "start",
     "stop",
@@ -46,6 +40,7 @@ __all__ = [
     "pop",
     "route",
     "add",
+    "request",
     "get",
     "post",
     "put",
